package jp.sagalab.skit;

import java.awt.Color;
import java.awt.Container;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.imageio.ImageIO;
import jp.sagalab.jflib.GestureEvent;
import jp.sagalab.jftk.curve.SplineCurve;
import org.apache.batik.dom.svg.SAXSVGDocumentFactory;
import org.apache.batik.util.XMLResourceDescriptor;
import org.w3c.dom.Document;

/**
 * ジェスチャーレイヤー。
 * @author kaneko
 */
public class GestureLayer extends Container {
	private static final int GESTURE_ANIMATION_FPS = 5;
	private static final int GESTURE_ANIMATION_GRADATION = 3;
	private static final String IMAGE_DIRECTORY_PATH = new File(".").getAbsoluteFile().getParent()+File.separator+"images"+File.separator;
	private static final Document SYMBOL_L_DELETE_D_SVG_DOCUMENT = readSVGDocument( GestureEvent.GestureType.L_SCRATCH_D );
	private static final Document SYMBOL_L_DELETE_DL_SVG_DOCUMENT = readSVGDocument( GestureEvent.GestureType.L_SCRATCH_DL );
	private static final Document SYMBOL_L_DELETE_DR_SVG_DOCUMENT = readSVGDocument( GestureEvent.GestureType.L_SCRATCH_DR );
	private static final Document SYMBOL_L_DELETE_L_SVG_DOCUMENT = readSVGDocument( GestureEvent.GestureType.L_SCRATCH_L );
	private static final Document SYMBOL_L_DELETE_R_SVG_DOCUMENT = readSVGDocument( GestureEvent.GestureType.L_SCRATCH_R );
	private static final Document SYMBOL_L_DELETE_U_SVG_DOCUMENT = readSVGDocument( GestureEvent.GestureType.L_SCRATCH_U );
	private static final Document SYMBOL_L_DELETE_UL_SVG_DOCUMENT = readSVGDocument( GestureEvent.GestureType.L_SCRATCH_UL );
	private static final Document SYMBOL_L_DELETE_UR_SVG_DOCUMENT = readSVGDocument( GestureEvent.GestureType.L_SCRATCH_UR );
	private static final Document SYMBOL_R_DELETE_D_SVG_DOCUMENT = readSVGDocument( GestureEvent.GestureType.R_SCRATCH_D );
	private static final Document SYMBOL_R_DELETE_DL_SVG_DOCUMENT = readSVGDocument( GestureEvent.GestureType.R_SCRATCH_DL );
	private static final Document SYMBOL_R_DELETE_DR_SVG_DOCUMENT = readSVGDocument( GestureEvent.GestureType.R_SCRATCH_DR );
	private static final Document SYMBOL_R_DELETE_L_SVG_DOCUMENT = readSVGDocument( GestureEvent.GestureType.R_SCRATCH_L );
	private static final Document SYMBOL_R_DELETE_R_SVG_DOCUMENT = readSVGDocument( GestureEvent.GestureType.R_SCRATCH_R );
	private static final Document SYMBOL_R_DELETE_U_SVG_DOCUMENT = readSVGDocument( GestureEvent.GestureType.R_SCRATCH_U );
	private static final Document SYMBOL_R_DELETE_UL_SVG_DOCUMENT = readSVGDocument( GestureEvent.GestureType.R_SCRATCH_UL );
	private static final Document SYMBOL_R_DELETE_UR_SVG_DOCUMENT = readSVGDocument( GestureEvent.GestureType.R_SCRATCH_UR );
	private static final BufferedImage COMMAND_DELETE_IMAGE = readGestureCommandImage( GestureEvent.GestureType.R_SCRATCH_D );
	private static final Document SYMBOL_F_REVERCE_D_SVG_DOCUMENT = readSVGDocument( GestureEvent.GestureType.F_REVERCE_D );
	private static final Document SYMBOL_F_REVERCE_DL_SVG_DOCUMENT = readSVGDocument( GestureEvent.GestureType.F_REVERCE_DL );
	private static final Document SYMBOL_F_REVERCE_DR_SVG_DOCUMENT = readSVGDocument( GestureEvent.GestureType.F_REVERCE_DR );
	private static final Document SYMBOL_F_REVERCE_L_SVG_DOCUMENT = readSVGDocument( GestureEvent.GestureType.F_REVERCE_L );
	private static final Document SYMBOL_F_REVERCE_R_SVG_DOCUMENT = readSVGDocument( GestureEvent.GestureType.F_REVERCE_R );
	private static final Document SYMBOL_F_REVERCE_U_SVG_DOCUMENT = readSVGDocument( GestureEvent.GestureType.F_REVERCE_U );
	private static final Document SYMBOL_F_REVERCE_UL_SVG_DOCUMENT = readSVGDocument( GestureEvent.GestureType.F_REVERCE_UL );
	private static final Document SYMBOL_F_REVERCE_UR_SVG_DOCUMENT = readSVGDocument( GestureEvent.GestureType.F_REVERCE_UR );
	private static final Document SYMBOL_R_REVERCE_D_SVG_DOCUMENT = readSVGDocument( GestureEvent.GestureType.R_REVERCE_D );
	private static final Document SYMBOL_R_REVERCE_DL_SVG_DOCUMENT = readSVGDocument( GestureEvent.GestureType.R_REVERCE_DL );
	private static final Document SYMBOL_R_REVERCE_DR_SVG_DOCUMENT = readSVGDocument( GestureEvent.GestureType.R_REVERCE_DR );
	private static final Document SYMBOL_R_REVERCE_L_SVG_DOCUMENT = readSVGDocument( GestureEvent.GestureType.R_REVERCE_L );
	private static final Document SYMBOL_R_REVERCE_R_SVG_DOCUMENT = readSVGDocument( GestureEvent.GestureType.R_REVERCE_R );
	private static final Document SYMBOL_R_REVERCE_U_SVG_DOCUMENT = readSVGDocument( GestureEvent.GestureType.R_REVERCE_U );
	private static final Document SYMBOL_R_REVERCE_UL_SVG_DOCUMENT = readSVGDocument( GestureEvent.GestureType.R_REVERCE_UL );
	private static final Document SYMBOL_R_REVERCE_UR_SVG_DOCUMENT = readSVGDocument( GestureEvent.GestureType.R_REVERCE_UR );
	private static final BufferedImage COMMAND_REVERCE_IMAGE = readGestureCommandImage( GestureEvent.GestureType.R_REVERCE_D );
	private static final Document SYMBOL_REDO_SVG_DOCUMENT = readSVGDocument( GestureEvent.GestureType.REDO );
	private static final BufferedImage COMMAND_REDO_IMAGE = readGestureCommandImage( GestureEvent.GestureType.REDO );
	private static final Document SYMBOL_UNDO_SVG_DOCUMENT = readSVGDocument( GestureEvent.GestureType.UNDO );
	private static final BufferedImage COMMAND_UNDO_IMAGE = readGestureCommandImage( GestureEvent.GestureType.UNDO );
	private static final Document SYMBOL_DIVIDE_SVG_DOCUMENT = readSVGDocument( GestureEvent.GestureType.SEPARATE );
	private static final BufferedImage COMMAND_SEPARATE_IMAGE = readGestureCommandImage( GestureEvent.GestureType.SEPARATE );
	private static final Document SYMBOL_UNKNOWN_DOCUMENT = readSVGDocument( GestureEvent.GestureType.UNKNOWN );
	private static final BufferedImage COMMAND_UNKNOWN_IMAGE = readGestureCommandImage( GestureEvent.GestureType.UNKNOWN );

	public static GestureLayer create( int _width, int _height ) {
		if ( _width <= 0 ) {
			throw new IllegalArgumentException( "_width <= 0" );
		}
		if ( _height <= 0 ) {
			throw new IllegalArgumentException( "_height <= 0" );
		}
		return new GestureLayer( _width, _height );
	}

	private GestureLayer( int _width, int _height ) {
		setBackground( new Color( 0, 0, 0, 0 ) );
		setSize( _width, _height );
	}

	public AnimationData addGesture( GestureEvent _gesture ) {
		if ( _gesture == null ) {
			throw new NullPointerException( "_gesture is null." );
		}
		Document svgDoc;
		BufferedImage command;
		switch ( _gesture.type() ) {
			case R_SCRATCH_D:
				svgDoc = SYMBOL_R_DELETE_D_SVG_DOCUMENT;
				command = COMMAND_DELETE_IMAGE;
				break;
			case R_SCRATCH_DL:
				svgDoc = SYMBOL_R_DELETE_DL_SVG_DOCUMENT;
				command = COMMAND_DELETE_IMAGE;
				break;
			case R_SCRATCH_DR:
				svgDoc = SYMBOL_R_DELETE_DR_SVG_DOCUMENT;
				command = COMMAND_DELETE_IMAGE;
				break;
			case R_SCRATCH_L:
				svgDoc = SYMBOL_R_DELETE_L_SVG_DOCUMENT;
				command = COMMAND_DELETE_IMAGE;
				break;
			case R_SCRATCH_R:
				svgDoc = SYMBOL_R_DELETE_R_SVG_DOCUMENT;
				command = COMMAND_DELETE_IMAGE;
				break;
			case R_SCRATCH_U:
				svgDoc = SYMBOL_R_DELETE_U_SVG_DOCUMENT;
				command = COMMAND_DELETE_IMAGE;
				break;
			case R_SCRATCH_UL:
				svgDoc = SYMBOL_R_DELETE_UL_SVG_DOCUMENT;
				command = COMMAND_DELETE_IMAGE;
				break;
			case R_SCRATCH_UR:
				svgDoc = SYMBOL_R_DELETE_UR_SVG_DOCUMENT;
				command = COMMAND_DELETE_IMAGE;
				break;
			case L_SCRATCH_D:
				svgDoc = SYMBOL_L_DELETE_D_SVG_DOCUMENT;
				command = COMMAND_DELETE_IMAGE;
				break;
			case L_SCRATCH_DL:
				svgDoc = SYMBOL_L_DELETE_DL_SVG_DOCUMENT;
				command = COMMAND_DELETE_IMAGE;
				break;
			case L_SCRATCH_DR:
				svgDoc = SYMBOL_L_DELETE_DR_SVG_DOCUMENT;
				command = COMMAND_DELETE_IMAGE;
				break;
			case L_SCRATCH_L:
				svgDoc = SYMBOL_L_DELETE_L_SVG_DOCUMENT;
				command = COMMAND_DELETE_IMAGE;
				break;
			case L_SCRATCH_R:
				svgDoc = SYMBOL_L_DELETE_R_SVG_DOCUMENT;
				command = COMMAND_DELETE_IMAGE;
				break;
			case L_SCRATCH_U:
				svgDoc = SYMBOL_L_DELETE_U_SVG_DOCUMENT;
				command = COMMAND_DELETE_IMAGE;
				break;
			case L_SCRATCH_UL:
				svgDoc = SYMBOL_L_DELETE_UL_SVG_DOCUMENT;
				command = COMMAND_DELETE_IMAGE;
				break;
			case L_SCRATCH_UR:
				svgDoc = SYMBOL_L_DELETE_UR_SVG_DOCUMENT;
				command = COMMAND_DELETE_IMAGE;
				break;
			case R_REVERCE_D:
				svgDoc = SYMBOL_R_REVERCE_D_SVG_DOCUMENT;
				command = COMMAND_REVERCE_IMAGE;
				break;
			case R_REVERCE_DL:
				svgDoc = SYMBOL_R_REVERCE_DL_SVG_DOCUMENT;
				command = COMMAND_REVERCE_IMAGE;
				break;
			case R_REVERCE_DR:
				svgDoc = SYMBOL_R_REVERCE_DR_SVG_DOCUMENT;
				command = COMMAND_REVERCE_IMAGE;
				break;
			case R_REVERCE_L:
				svgDoc = SYMBOL_R_REVERCE_L_SVG_DOCUMENT;
				command = COMMAND_REVERCE_IMAGE;
				break;
			case R_REVERCE_R:
				svgDoc = SYMBOL_R_REVERCE_R_SVG_DOCUMENT;
				command = COMMAND_REVERCE_IMAGE;
				break;
			case R_REVERCE_U:
				svgDoc = SYMBOL_R_REVERCE_U_SVG_DOCUMENT;
				command = COMMAND_REVERCE_IMAGE;
				break;
			case R_REVERCE_UL:
				svgDoc = SYMBOL_R_REVERCE_UL_SVG_DOCUMENT;
				command = COMMAND_REVERCE_IMAGE;
				break;
			case R_REVERCE_UR:
				svgDoc = SYMBOL_R_REVERCE_UR_SVG_DOCUMENT;
				command = COMMAND_REVERCE_IMAGE;
				break;
			case F_REVERCE_D:
				svgDoc = SYMBOL_F_REVERCE_D_SVG_DOCUMENT;
				command = COMMAND_REVERCE_IMAGE;
				break;
			case F_REVERCE_DL:
				svgDoc = SYMBOL_F_REVERCE_DL_SVG_DOCUMENT;
				command = COMMAND_REVERCE_IMAGE;
				break;
			case F_REVERCE_DR:
				svgDoc = SYMBOL_F_REVERCE_DR_SVG_DOCUMENT;
				command = COMMAND_REVERCE_IMAGE;
				break;
			case F_REVERCE_L:
				svgDoc = SYMBOL_F_REVERCE_L_SVG_DOCUMENT;
				command = COMMAND_REVERCE_IMAGE;
				break;
			case F_REVERCE_R:
				svgDoc = SYMBOL_F_REVERCE_R_SVG_DOCUMENT;
				command = COMMAND_REVERCE_IMAGE;
				break;
			case F_REVERCE_U:
				svgDoc = SYMBOL_F_REVERCE_U_SVG_DOCUMENT;
				command = COMMAND_REVERCE_IMAGE;
				break;
			case F_REVERCE_UL:
				svgDoc = SYMBOL_F_REVERCE_UL_SVG_DOCUMENT;
				command = COMMAND_REVERCE_IMAGE;
				break;
			case F_REVERCE_UR:
				svgDoc = SYMBOL_F_REVERCE_UR_SVG_DOCUMENT;
				command = COMMAND_REVERCE_IMAGE;
				break;
			case UNDO:
				svgDoc = SYMBOL_UNDO_SVG_DOCUMENT;
				command = COMMAND_UNDO_IMAGE;
				break;
			case REDO:
				svgDoc = SYMBOL_REDO_SVG_DOCUMENT;
				command = COMMAND_REDO_IMAGE;
				break;
			case SEPARATE:
				svgDoc = SYMBOL_DIVIDE_SVG_DOCUMENT;
				command = COMMAND_SEPARATE_IMAGE;
				break;
			default:
				svgDoc = SYMBOL_UNKNOWN_DOCUMENT;
				command = COMMAND_UNKNOWN_IMAGE;
				break;
		}
		SplineCurve fsc = _gesture.fsc();
		GestureAnimationData data = GestureAnimationData.create( fsc, svgDoc, command, GESTURE_ANIMATION_GRADATION, GESTURE_ANIMATION_FPS );

		add( data );

		return data;
	}

	public static Document readSVGDocument( GestureEvent.GestureType _type ) {
		{
			String parser = XMLResourceDescriptor.getXMLParserClassName();
			SAXSVGDocumentFactory f = new SAXSVGDocumentFactory( parser );
			Document doc = null;
			try {
				doc = f.createDocument( new File( IMAGE_DIRECTORY_PATH + getSymbolSVGFileName( _type ) ).toURI().toString() );
			} catch ( IOException ex ) {
				Logger.getLogger( GestureLayer.class.getName() ).log( Level.SEVERE, null, ex );
			}
			return doc;
		}
	}

	private static BufferedImage readGestureCommandImage( GestureEvent.GestureType _type ) {
		BufferedImage image = null;
		try {
			image = ImageIO.read( new File( IMAGE_DIRECTORY_PATH + getCommandImageFileName( _type ) ) );
		} catch ( IOException ex ) {
			Logger.getLogger( GestureLayer.class.getName() ).log( Level.SEVERE, null, ex );
		}
		return image;
	}

	private static String getSymbolSVGFileName( GestureEvent.GestureType _type ) {
		String symbolImageFileName;
		switch ( _type ) {
			case L_SCRATCH_D:
				symbolImageFileName = "scratch8.svg";
				break;
			case L_SCRATCH_DL:
				symbolImageFileName = "scratch7.svg";
				break;
			case L_SCRATCH_DR:
				symbolImageFileName = "scratch1.svg";
				break;
			case L_SCRATCH_L:
				symbolImageFileName = "scratch6.svg";
				break;
			case L_SCRATCH_R:
				symbolImageFileName = "scratch2.svg";
				break;
			case L_SCRATCH_U:
				symbolImageFileName = "scratch4.svg";
				break;
			case L_SCRATCH_UL:
				symbolImageFileName = "scratch5.svg";
				break;
			case L_SCRATCH_UR:
				symbolImageFileName = "scratch3.svg";
				break;
			case R_SCRATCH_D:
				symbolImageFileName = "scratch_rv8.svg";
				break;
			case R_SCRATCH_DL:
				symbolImageFileName = "scratch_rv7.svg";
				break;
			case R_SCRATCH_DR:
				symbolImageFileName = "scratch_rv1.svg";
				break;
			case R_SCRATCH_L:
				symbolImageFileName = "scratch_rv6.svg";
				break;
			case R_SCRATCH_R:
				symbolImageFileName = "scratch_rv2.svg";
				break;
			case R_SCRATCH_U:
				symbolImageFileName = "scratch_rv4.svg";
				break;
			case R_SCRATCH_UL:
				symbolImageFileName = "scratch_rv5.svg";
				break;
			case R_SCRATCH_UR:
				symbolImageFileName = "scratch_rv3.svg";
				break;
			case F_REVERCE_D:
				symbolImageFileName = "direction3.svg";
				break;
			case F_REVERCE_DL:
				symbolImageFileName = "direction2.svg";
				break;
			case F_REVERCE_DR:
				symbolImageFileName = "direction4.svg";
				break;
			case F_REVERCE_L:
				symbolImageFileName = "direction1.svg";
				break;
			case F_REVERCE_R:
				symbolImageFileName = "direction_rv5.svg";
				break;
			case F_REVERCE_U:
				symbolImageFileName = "direction_rv7.svg";
				break;
			case F_REVERCE_UL:
				symbolImageFileName = "direction_rv8.svg";
				break;
			case F_REVERCE_UR:
				symbolImageFileName = "direction_rv6.svg";
				break;
			case R_REVERCE_D:
				symbolImageFileName = "direction_rv3.svg";
				break;
			case R_REVERCE_DL:
				symbolImageFileName = "direction_rv2.svg";
				break;
			case R_REVERCE_DR:
				symbolImageFileName = "direction_rv4.svg";
				break;
			case R_REVERCE_L:
				symbolImageFileName = "direction_rv1.svg";
				break;
			case R_REVERCE_R:
				symbolImageFileName = "direction5.svg";
				break;
			case R_REVERCE_U:
				symbolImageFileName = "direction7.svg";
				break;
			case R_REVERCE_UL:
				symbolImageFileName = "direction8.svg";
				break;
			case R_REVERCE_UR:
				symbolImageFileName = "direction6.svg";
				break;
			case UNDO:
				symbolImageFileName = "undo.svg";
				break;
			case REDO:
				symbolImageFileName = "redo.svg";
				break;
			case SEPARATE:
				symbolImageFileName = "divide.svg";
				break;
			default:
				symbolImageFileName = "unknown.svg";
				break;
		}
		return symbolImageFileName;
	}

	private static String getCommandImageFileName( GestureEvent.GestureType _type ) {
		String commandImageFileName;
		switch ( _type ) {
			case R_SCRATCH_D:
				commandImageFileName = "command_delete.png";
				break;
			case R_REVERCE_D:
				commandImageFileName = "command_reverse.png";
				break;
			case UNDO:
				commandImageFileName = "command_undo.png";
				break;
			case REDO:
				commandImageFileName = "command_redo.png";
				break;
			case SEPARATE:
				commandImageFileName = "command_separate.png";
				break;
			default:
				commandImageFileName = "command_unknown.png";
				break;
		}
		return commandImageFileName;
	}
}
