package jp.sagalab.skit;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;

/**
 * グリッドレイヤー。
 * @author kaneko
 */
public class GridLayer extends Component {
	private static final Color GRID_COLOR = new Color( 255,255,255,25 );
	private final ReadOnlySKITModel m_model;

	public static GridLayer create( ReadOnlySKITModel _model, int _width, int _height ) {
		if ( _model == null ) {
			throw new NullPointerException( "_model is null" );
		}
		if ( _width <= 0 ) {
			throw new IllegalArgumentException( "_width <= 0" );
		}
		if ( _height <= 0 ) {
			throw new IllegalArgumentException( "_height <= 0" );
		}
		return new GridLayer( _model, _width, _height );
	}

	private GridLayer( ReadOnlySKITModel _model, int _width, int _height ) {
		m_model = _model;
		setBackground( new Color( 0, 0, 0, 0 ) );
		setSize( _width, _height );
	}

	@Override
	public void paint( Graphics _g ) {
		Graphics2D g = (Graphics2D) _g;
		Dimension dimension = getSize();

		// グリッドの描画
		g.setColor( GRID_COLOR );
		SKITPanelHelper.drawBasedGrid( g, m_model.getGridSpace(),-2, m_model.getViewToModel(), dimension.getWidth(), dimension.getHeight());
		SKITPanelHelper.drawBasedGrid( g, m_model.getGridSpace(),0, m_model.getViewToModel(), dimension.getWidth(), dimension.getHeight());
	}
}
