package jp.sagalab.skit;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Stroke;
import java.util.List;
import jp.sagalab.jflib.Shape;
import jp.sagalab.jftk.curve.primitive.PrimitiveCurve;
import jp.sagalab.jftk.transform.TransformMatrix;

/**
 *
 * @author kaneko
 */
public class GuideLineLayer extends Component {
	private static final Color GUIDELINE_COLOR = new Color( 233, 195, 88, 255 );
	private static final Stroke GUIDELINE_STROKE = new BasicStroke( 1.3f );
	private final ReadOnlySKITModel m_model;

	public static GuideLineLayer create( ReadOnlySKITModel _model, int _width, int _height ) {
		if ( _model == null ) {
			throw new NullPointerException( "_model is null." );
		}
		if ( _width <= 0 ) {
			throw new IllegalArgumentException( "_width <= 0" );
		}
		if ( _height <= 0 ) {
			throw new IllegalArgumentException( "_height <= 0" );
		}
		return new GuideLineLayer( _model, _width, _height );
	}

	private GuideLineLayer( ReadOnlySKITModel _model, int _width, int _height ) {
		m_model = _model;
		setBackground( new Color( 0, 0, 0, 0 ) );
		setSize( _width, _height );
	}

	@Override
	public void paint( Graphics _g ) {
		Graphics2D g = (Graphics2D) _g;

		drawShapeGuideLines( g, m_model.getHotShapes(), m_model.getViewToModel().inverse() );
	}

	private static void drawShapeGuideLines( Graphics2D _g, List<Shape> _shapes, TransformMatrix _modelToView ) {
		for ( Shape shape : _shapes ) {
			_g.setColor( GUIDELINE_COLOR );
			_g.setStroke( GUIDELINE_STROKE );
			for ( PrimitiveCurve primitive : shape.primitives() ) {
				PrimitiveCurve transed = primitive.transform( _modelToView );
				// ガイド線描画
				SKITPanelHelper.drawGuideline( _g, transed, shape.snappingTargetDataMap().get( primitive ), _modelToView );
			}
		}
		}
	}
