package jp.sagalab.skit;

import java.awt.AWTException;
import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.Robot;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;

/**
 *
 * @author kaneko
 */
public class ImageCreater {
	private ImageCreater() {
		throw new UnsupportedOperationException();
	}

	/**
	 * 透過レイヤー画像を生成する。
	 * @param _width 幅。
	 * @param _height 高さ。
	 * @return 透過レイヤー画像。
	 */
	public static BufferedImage createTransparentImage( int _width, int _height ) {
		BufferedImage image = new BufferedImage( _width, _height, BufferedImage.TYPE_INT_ARGB );
		Graphics2D g = image.createGraphics();
		g.setComposite( AlphaComposite.getInstance( AlphaComposite.CLEAR, 0.0f ) );
		Rectangle2D.Double rectangle = new Rectangle2D.Double( 0, 0, _width, _height );
		g.fill( rectangle );
		g.dispose();
		return image;
	}

	public static BufferedImage createBackgroundImage( int _x, int _y, int _width, int _height ) throws AWTException {
		Rectangle bound = new Rectangle( _x, _y, _width, _height );
		// 背景画像をキャプチャ
		BufferedImage bgImage = createBackgroundImage( bound );

		return bgImage;
	}

	/**
	 * 擬似透明フレームのためのスクリーンキャプチャ取得
	 * @param キャプチャ領域
	 */
	private static BufferedImage createBackgroundImage( Rectangle _bound ) throws AWTException {
		// スクリーンキャプチャを取得
		BufferedImage image = new Robot().createScreenCapture( _bound );

		// 画像を薄暗くする。
		BufferedImage screened= new BufferedImage(_bound.width, _bound.height, BufferedImage.TYPE_INT_ARGB );
		for ( int y = 0; y < image.getHeight(); y++ ) {
			for ( int x = 0; x < image.getWidth(); x++ ) {
				Color c= new Color(image.getRGB( x, y));
				float[] rgb= c.getRGBColorComponents( new float[ 3] );
				//薄暗くする 0〜1
				float ratio=0.45f;
				screened.setRGB( x, y, new Color(rgb[0]*ratio,rgb[1]*ratio,rgb[2]*ratio).getRGB() );
			}
		}
		return screened;
	}

	/**
	 * 指定されたイメージをコピーする。
	 * @param _image イメージ
	 * @return コピーイメージ
	 */
	public static BufferedImage copyImage( BufferedImage _image ) {
		BufferedImage newImage = new BufferedImage( _image.getWidth(), _image.getHeight(), _image.getType() );
		Graphics2D g = newImage.createGraphics();
		g.drawImage( _image, 0, 0, null );
		g.dispose();
		return newImage;
	}
}
