package jp.sagalab.skit;

import jp.sagalab.jftk.transform.TransformMatrix;

/**
 * モデル空間情報
 * @author hasegawa
 */
// TODO SimMatrixにする?
public class ModelSpaceInfo {
	/** 変換行列 */
	private TransformMatrix m_mat;
	/** グリッド空間情報 */
	private TransformMatrix m_grid;
	/** 派生倍率 */
	private int m_derivatinalScaleFactor;

	/**
	 * コンストラクタ
	 */
	public ModelSpaceInfo() {
		double[][] elements = new double[ 4 ][ 4 ];
		for ( int i = 0; i < 4; ++i ) {
			for ( int j = 0; j < 4; ++j ) {
				elements[i][j] = 0.0;
			}
		}
		m_mat = TransformMatrix.create(elements );
		m_grid = TransformMatrix.create(elements );
	}

	/**
	 * コンストラクタ
	 * @param _mat
	 * @param _grid
	 */
	public ModelSpaceInfo( TransformMatrix _mat, TransformMatrix _grid ) {
		m_mat = _mat;
		m_grid = _grid;
	}

	/**
	 * 変換行列の取得
	 * @return
	 */
	public TransformMatrix mappingMatrix() {
		return m_mat;
	}

	/**
	 * 変換行列の設定
	 * @param _mat
	 */
	public void setMappingMatrix( TransformMatrix _mat ) {
		m_mat = _mat;
	}

	/**
	 * グリッド空間情報の取得
	 * @return
	 */
	public TransformMatrix gridSpaceMatrix() {
		return m_grid;
	}

	/**
	 * 派生倍率の取得
	 * @return 派生倍率
	 */
	public int derivatinalScaleFactor() {
		return m_derivatinalScaleFactor;
	}

	/**
	 * グリッド空間情報の設定
	 * @param _grid
	 */
	public void setGridSpaceMatrix( TransformMatrix _grid ) {
		m_grid = _grid;
	}

	/**
	 * 派生倍率の設定
	 * @param _derivatinalScaleFactor 派生倍率
	 */
	public void setDerivatinalScaleFactor( int _derivatinalScaleFactor ) {
		m_derivatinalScaleFactor = _derivatinalScaleFactor;
	}
}
