package jp.sagalab.skit;

import java.util.List;
import jp.sagalab.jflib.Shape;
import jp.sagalab.jftk.Point;
import jp.sagalab.jftk.snapper.GridSpace;
import jp.sagalab.jftk.transform.TransformMatrix;

/**
 * モデルのパラメータを読み取るインタフェース。
 * @author oyoshi
 */
public interface ReadOnlySKITModel {
	/**
	 * モデルに登録されている図形群を返す。
	 * @return モデルに登録されている図形群。
	 */
	List<Shape> getShapes();

	/**
	 * ホットな図形を返す。
	 * @return ホットな図形。
	 */
	List<Shape> getHotShapes();

	/**
	 * アクティブな図形を返す。
	 * @return アクティブな図形。
	 */
	List<Shape> getActiveShapes();

	/**
	 * 非アクティブな図形を返す。
	 * @return 非アクティブな図形。
	 */
	List<Shape> getInactiveShapes();

	/**
	 * モデルに登録されている絶対図形空間操作における図形群の特徴点列を返す。
	 * @return 特徴点列。
	 */
	Point[] getFeaturePoints();
	
	/**
	 * モデルに登録されている絶対図形空間操作における固定点のインデックスを返す。
	 * @return 固定点のインデックス。
	 */
	int getPivotPointIndex();
	
	/**
	 * モデルに登録されている絶対図形空間操作における選択点のインデックスを返す。
	 * @return 選択点のインデックス。
	 */
	int getSelectedPointIndex();

	/**
	 * 初期のビューポート空間からモデル空間への変換行列を返す。
	 * @return 初期のビューポート空間からモデル空間への変換行列。
	 */
	TransformMatrix getInitialViewToModel();
	
	/**
	 * 初期の単位グリッドからモデル空間中の直交グリッドへの変換行列を返す。
	 * @return 初期の単位グリッドからモデル空間中の直交グリッドへの変換行列。
	 */
	GridSpace getInitialGridSpace();
	/**
	 * ビューポート空間からモデル空間への変換行列を返す。
	 * @return ビューポート空間からモデル空間への変換行列。
	 */
	TransformMatrix getViewToModel();

	/**
	 * 単位グリッドからモデル空間中の直交グリッドへの変換行列を返す。
	 * @return 単位グリッドからモデル空間上の直交グリッドへの変換行列。
	 */
	GridSpace getGridSpace();
}
