package jp.sagalab.skit;

import jp.sagalab.jflib.GestureEventListener;
import jp.sagalab.jflib.Shape;
import jp.sagalab.jftk.snapper.GridSpace;
import jp.sagalab.jftk.Point;
import jp.sagalab.jftk.transform.TransformMatrix;

/**
 * SKITイベントを受信するためのイベントリスナー。
 */
public interface SKITEventListener extends GestureEventListener {
	/**
	 * アクティブ図形群が更新されたときに呼び出される。
	 * @param _added 追加されたアクティブ図形群。
	 * @param _removed 削除されたアクティブ図形群。
	 */
	void activeShapesChanged( Shape[] _added, Shape[] _removed );
	/**
	 * 非アクティブ図形群が更新されたときに呼び出される。
	 * @param _added 追加された非アクティブ図形群。
	 * @param _removed 削除された非アクティブ図形群。
	 */
	void inactiveShapesChanged( Shape[] _added, Shape[] _removed );

	/**
	 * グリッド空間が更新されたときに呼び出される。
	 * @param _grid 更新後のグリッド空間。
	 */
	void gridSpaceChanged( GridSpace _grid );

	/**
	 * モデル空間行列が更新されたときに呼び出される。
	 * @param _mat 更新後のモデル空間行列。
	 */
	void transformChanged( TransformMatrix _mat );
	
	/**
	 * 絶対図形空間操作における特徴点列が更新されたときに呼び出される。
	 * @param _featurePoints 特徴点列。
	 */
	void featurePointsChanged( Point[] _featurePoints );
	
	/**
	 * 絶対図形空間操作における固定点・選択点が更新されたときに呼び出される。
	 * @param _pivotIndex 固定点のインデックス。
	 * @param _selectedIndex 選択点のインデックス。
	 */
	void regardPointIndexChanged( int _pivotIndex, int _selectedIndex );
}
