package jp.sagalab.skit;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.logging.Level;
import java.util.logging.LogManager;
import java.util.logging.Logger;

/**
 * ログ設定。
 */
class SKITLogger {
	private static final String LOGGING_PROPERTIES 
		= "handlers=java.util.logging.ConsoleHandler, java.util.logging.FileHandler\n"
		+ ".level=INFO\n"
		+ "java.util.logging.ConsoleHandler.level=INFO\n"
		+ "java.util.logging.ConsoleHandler.formatter=java.util.logging.SimpleFormatter\n"
		+ "java.util.logging.FileHandler.level=INFO\n"
		+ "java.util.logging.FileHandler.pattern="+new File(".").getAbsoluteFile().getParentFile().getParentFile().getParentFile().getParentFile().getParent()+File.separator+"SKITErrorMessage%u.log\n"
		+ "java.util.logging.FileHandler.formatter=java.util.logging.SimpleFormatter\n"
		+ "java.util.logging.FileHandler.limit=50000"
		+ "java.util.logging.FileHandler.append=true";

	private SKITLogger() {
		throw new UnsupportedOperationException();
	}

	static void configure() {
		Logger logger = Logger.getLogger( SKITLogger.class.getName() );

		InputStream in = null;
		try {
			in = new ByteArrayInputStream( LOGGING_PROPERTIES.getBytes( "UTF-8" )) ;	
			LogManager.getLogManager().readConfiguration( in );
		} catch ( IOException ex ) {
			logger.log( Level.WARNING, null, ex );
		} finally {
			try {
				if ( in != null ) {
					in.close();
				}
			} catch ( IOException ex ) {
				logger.log( Level.WARNING, null, ex );
			}
		}
	}
}
