package jp.sagalab.skit;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.Stroke;
import java.awt.event.MouseEvent;
import java.awt.image.BufferedImage;
import javax.swing.event.MouseInputListener;

/**
 *
 * @author kaneko
 */
public class ScratchLayer extends Component implements MouseInputListener {
	private static final float INPUTS_WIDTH = 2f;
	private static final Color INPUTS_COLOR = Color.white;
	private static final Stroke INPUTS_STROKE = new BasicStroke( INPUTS_WIDTH );
	private final BufferedImage m_foreground;
	private BufferedImage m_scratchedForeground;
	/** 直近のイベントで入力された点のX座標値 */
	private int m_preX;
	/** 直近のイベントで入力された点のY座標値 */
	private int m_preY;

	public static ScratchLayer create( int _width, int _height ) {
		if ( _width <= 0 ) {
			throw new IllegalArgumentException( "_width <= 0" );
		}
		if ( _height <= 0 ) {
			throw new IllegalArgumentException( "_height <= 0" );
		}
		BufferedImage image = ImageCreater.createTransparentImage( _width, _height );
		return new ScratchLayer( image );
	}

	private ScratchLayer( BufferedImage _foreground ) {
		m_foreground = _foreground;
		m_scratchedForeground = ImageCreater.copyImage( _foreground );
		setBackground( new Color( 0, 0, 0, 0 ) );
		setSize( _foreground.getWidth(), _foreground.getHeight() );
	}

	@Override
	public void paint( Graphics _g ) {
		_g.drawImage( m_scratchedForeground, 0, 0, null );
	}

	@Override
	public void mousePressed( MouseEvent me ) {
		// マウスの第1ボタンのみ押している時
		if( me.getModifiers()== MouseEvent.BUTTON1_MASK ) {
			m_preX = me.getX();
			m_preY = me.getY();
			//左クリック押下
		}
	}

	@Override
	public void mouseReleased( MouseEvent me ) {
		// マウスの第1ボタンのみ押している時
		if( me.getModifiers()== MouseEvent.BUTTON1_MASK ) {
			// 前回追加した点と今回追加した点の間に線分を描画
			drawLine( m_preX, m_preY, me.getX(), me.getY() );
			// 表示用画像をクリアして再描画
			m_scratchedForeground = ImageCreater.copyImage( m_foreground );
			repaint();
		}
	}

	@Override
	public void mouseDragged( MouseEvent me ) {
		// マウスの第1ボタンのみ押している時
		if( me.getModifiers()== MouseEvent.BUTTON1_MASK ) {
			// 前回追加した点と今回追加した点の間に線分を描画
			drawLine( m_preX, m_preY, me.getX(), me.getY() );
			// 新たに描画した部分を再描画
			repaintDirtyArea( m_preX, m_preY, me.getX(), me.getY() );
			//前回追加した点の更新
			m_preX = me.getX();
			m_preY = me.getY();
		}
	}

	@Override
	public void mouseClicked( MouseEvent me ) {
	}

	@Override
	public void mouseEntered( MouseEvent me ) {
	}

	@Override
	public void mouseExited( MouseEvent me ) {
	}

	@Override
	public void mouseMoved( MouseEvent me ) {
	}

	private void drawLine( int _preX, int _preY, int _x, int _y ) {
		// 描画設定
		Graphics2D g = m_scratchedForeground.createGraphics();
		g.setRenderingHint( RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON );
		g.setStroke( INPUTS_STROKE );
		g.setColor( INPUTS_COLOR );
		g.drawLine( _preX, _preY, _x, _y );
		g.dispose();
	}

	private void repaintDirtyArea( int _preX, int _preY, int _x, int _y ) {
		// 再描画が必要な範囲の算出
		int x = Math.min( _preX, _x );
		int y = Math.min( _preY, _y );
		int w = Math.max( _preX, _x ) - x;
		int h = Math.max( _preY, _y ) - y;

		// ストロークの太さの分だけ範囲を拡張して再描画
		int strokeRadius = (int) Math.ceil( 0.5f * INPUTS_WIDTH );
		repaint( x - strokeRadius, y - strokeRadius, w + strokeRadius * 2, h + strokeRadius * 2 );
	}
}
