package jp.sagalab.skit;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.util.List;
import jp.sagalab.jflib.Shape;
import jp.sagalab.jftk.snapper.GridSpace;
import jp.sagalab.jftk.Point;
import jp.sagalab.jftk.snapper.SnappingTargetData;
import jp.sagalab.jftk.transform.TransformMatrix;

/**
 *
 * @author kaneko
 */
public class SnappingDataLayer extends Component {
	private static final Color SNAPPED_POINT_COLOR = new Color( 231,121, 31, 180 );
	private final ReadOnlySKITModel m_model;

	public static SnappingDataLayer create( ReadOnlySKITModel _model, int _width, int _height ) {
		if ( _model == null ) {
			throw new NullPointerException( "_model is null." );
		}
		if ( _width <= 0 ) {
			throw new IllegalArgumentException( "_width <= 0" );
		}
		if ( _height <= 0 ) {
			throw new IllegalArgumentException( "_height <= 0" );
		}
		return new SnappingDataLayer( _model, _width, _height );
	}

	private SnappingDataLayer( ReadOnlySKITModel _model, int _width, int _height ) {
		m_model = _model;
		setBackground( new Color( 0, 0, 0, 0 ) );
		setSize( _width, _height );
	}

	public void updateScene() {
	}

	@Override
	public void paint( Graphics _g ) {
		Graphics2D g = (Graphics2D) _g;

		TransformMatrix modelToView = m_model.getViewToModel().inverse();

		drawShapeSnappedPoints( g, m_model.getHotShapes(), modelToView );
	}

	private static void drawShapeSnappedPoints( Graphics2D _g, List<Shape> _shapes, TransformMatrix _modelToView ) {
		for ( Shape shape : _shapes ) {
			drawSnappedPoint( _g, shape.snappingTargetData(), _modelToView );
		}
	}

	/**
	 * スナッピングされた点を描画する。
	 * @param _g
	 * @param _snappingDataSet スナッピング情報群。
	 * @param _modelToView モデル空間からビューポート空間への変換行列
	 */
	private static void drawSnappedPoint( Graphics2D _g, List<SnappingTargetData> _snappingDataSet, TransformMatrix _modelToView ) {
		_g.setColor( SNAPPED_POINT_COLOR );
		for ( SnappingTargetData snappingTargetData : _snappingDataSet ) {
			Point[] snappedPoints = snappingTargetData.getSnappedPoints();
			GridSpace[] gridSpaces = snappingTargetData.getGridSpaceSet();
			int num = snappedPoints.length;
			for ( int i = 0; i < num; ++i ) {
				SKITPanelHelper.fillCircle( _g, snappedPoints[i].transform( _modelToView ), 10.0 );
				// スナッピングされたグリッドを描画
				SKITPanelHelper.drawGridBox( _g, snappedPoints[i], gridSpaces[i], _modelToView );
			}
		}
	}
	}
