package jp.sagalab.skit;

import jp.sagalab.jftk.snapper.GridSpace;
import jp.sagalab.jftk.Point;
import jp.sagalab.jftk.transform.TransformMatrix;

/**
 * モデルのパラメータへ書き込むインタフェース。
 * @author oyoshi
 */
public interface WriteOnlySKITModel {
	/**
	 * ビューポート空間からモデル空間への変換行列を設定。
	 * @param _mat ビューポート空間からモデル空間への変換行列。
	 */
	public void setViewToModel( TransformMatrix _mat ); 

	/**
	 * 単位グリッドからモデル空間中の直交グリッドへの変換行列を設定。
	 * @param _gridSpace 単位グリッドからモデル空間上の直交グリッドへの変換行列。
	 */
	public void setGridSpace( GridSpace _gridSpace ); 
	
	/**
	 * 絶対図形空間操作における特徴点列を設定。
	 * @param _points 特徴点列。
	 */
	void setFeaturePoints( Point[] _points );
	
	/**
	 * 絶対図形空間操作における固定点・選択点のインデックスを設定。
	 * @param _pivotIndex　固定点。
	 * @param _selectedIndex 選択点。
	 */
	void setRegardPointIndexes( int _pivotIndex, int _selectedIndex );
	
	/**
	 * 絶対図形空間操作における特徴点列を初期化。
	 */
	void resetFeaturePoints();
}
